@@ -147,6 +147,9 @@ class MySQL_Data_Stream
 
 	uint8_t pkt_sid;
 
+	bool com_field_list;
+	char *com_field_wild;
+
 	MySQL_Data_Stream();
 	~MySQL_Data_Stream();
 
@@ -203,13 +206,7 @@ class MySQL_Data_Stream
 
 	void return_MySQL_Connection_To_Pool();
 	
-	void destroy_MySQL_Connection_From_Pool(bool sq) {
-		MySQL_Connection *mc=myconn;
-		detach_connection();
-		unplug_backend();
-		mc->send_quit=sq;
-		MyHGM->destroy_MyConn_from_pool(mc);
-	}
+	void destroy_MySQL_Connection_From_Pool(bool sq);
 	void free_mysql_real_query();	
 	void reinit_queues();
 	void destroy_queues();
