@@ -46,6 +46,12 @@ class Query_Info {
 
 	int QueryLength;
 	enum MYSQL_COM_QUERY_command MyComQueryCmd;
+	bool bool_is_select_NOT_for_update;
+	bool bool_is_select_NOT_for_update_computed;
+	bool have_affected_rows;
+	uint64_t affected_rows;
+	uint64_t rows_sent;
+	uint64_t waiting_since;
 
 	Query_Info();
 	~Query_Info();
@@ -84,7 +90,7 @@ class MySQL_Session
 	void handler___status_WAITING_CLIENT_DATA___STATE_SLEEP___MYSQL_COM_SET_OPTION(PtrSize_t *);
 	void handler___status_WAITING_CLIENT_DATA___STATE_SLEEP___MYSQL_COM_STATISTICS(PtrSize_t *);
 	void handler___status_WAITING_CLIENT_DATA___STATE_SLEEP___MYSQL_COM_PROCESS_KILL(PtrSize_t *);
-	bool handler___status_WAITING_CLIENT_DATA___STATE_SLEEP___MYSQL_COM_QUERY_qpo(PtrSize_t *, bool ps=false);
+	bool handler___status_WAITING_CLIENT_DATA___STATE_SLEEP___MYSQL_COM_QUERY_qpo(PtrSize_t *, bool *lock_hostgroup, bool ps=false);
 
 	void handler___client_DSS_QUERY_SENT___server_DSS_NOT_INITIALIZED__get_connection();	
 
@@ -93,6 +99,7 @@ class MySQL_Session
 	bool handler_CommitRollback(PtrSize_t *);
 	bool handler_SetAutocommit(PtrSize_t *);
 	void RequestEnd(MySQL_Data_Stream *);
+	void LogQuery(MySQL_Data_Stream *);
 
 	void handler___status_WAITING_CLIENT_DATA___STATE_SLEEP___MYSQL_COM_QUERY___create_mirror_session();
 	int handler_again___status_PINGING_SERVER();
@@ -121,10 +128,6 @@ class MySQL_Session
 	void reset();
 	void add_ldap_comment_to_pkt(PtrSize_t *);
 
-	//this pointer is always initialized inside handler().
-	// it is an attempt to start simplifying the complexing of handler()
-	PtrSize_t *pktH;
-
 
 	public:
 	void * operator new(size_t);
@@ -149,6 +152,10 @@ class MySQL_Session
 	MySQL_Data_Stream *server_myds;
 	char * default_schema;
 
+	//this pointer is always initialized inside handler().
+	// it is an attempt to start simplifying the complexing of handler()
+	PtrSize_t *pktH;
+
 	uint32_t thread_session_id;
 	unsigned long long last_insert_id;
 	int last_HG_affected_rows;
@@ -157,6 +164,7 @@ class MySQL_Session
 	int user_max_connections;
 	int current_hostgroup;
 	int default_hostgroup;
+	int locked_on_hostgroup;
 	int next_query_flagIN;
 	int mirror_hostgroup;
 	int mirror_flagOUT;
@@ -184,6 +192,10 @@ class MySQL_Session
 
 	bool with_gtid;
 
+	char gtid_buf[128];
+	//uint64_t gtid_trxid;
+	int gtid_hid;
+
 	MySQL_STMTs_meta *sess_STMTs_meta;
 	StmtLongDataHandler *SLDH;
 
@@ -231,6 +243,7 @@ class MySQL_Session
 	bool handle_command_query_kill(PtrSize_t *);
 	void finishQuery(MySQL_Data_Stream *myds, MySQL_Connection *myconn, bool);
 	void generate_proxysql_internal_session_json(json &);
+	bool known_query_for_locked_on_hostgroup(uint64_t);
 };
 
 #define KILL_QUERY       1
