@@ -228,6 +228,8 @@ class MySQL_Thread
 		unsigned long long queries_slow;
 		unsigned long long queries_gtid;
 		unsigned long long queries_with_max_lag_ms;
+		unsigned long long queries_with_max_lag_ms__delayed;
+		unsigned long long queries_with_max_lag_ms__total_wait_time_us;
 		unsigned long long queries_backends_bytes_sent;
 		unsigned long long queries_backends_bytes_recv;
 		unsigned long long queries_frontends_bytes_sent;
@@ -251,6 +253,9 @@ class MySQL_Thread
 		unsigned long long unexpected_packet;
 		unsigned long long killed_connections;
 		unsigned long long killed_queries;
+		unsigned long long hostgroup_locked;
+		unsigned long long hostgroup_locked_set_cmds;
+		unsigned long long hostgroup_locked_queries;
 		unsigned long long aws_aurora_replicas_skipped_during_query;
 		unsigned int active_transactions;
 	} status_variables;
@@ -391,6 +396,7 @@ class MySQL_Threads_Handler
 		bool commands_stats;
 		bool query_digests;
 		bool query_digests_lowercase;
+		bool query_digests_replace_null;
 		bool query_digests_normalize_digest_text;
 		bool query_digests_track_hostname;
 		bool default_reconnect;
@@ -424,6 +430,7 @@ class MySQL_Threads_Handler
 		int default_query_timeout;
 		int query_processor_iterations;
 		int query_processor_regex;
+		int set_query_lock_on_hostgroup;
 		int reset_connection_algorithm;
 		int auto_increment_delay_multiplex;
 		int long_query_time;
@@ -443,6 +450,8 @@ class MySQL_Threads_Handler
 		int connpoll_reset_queue_length;
 		char *eventslog_filename;
 		int eventslog_filesize;
+		int eventslog_default_log;
+		int eventslog_format;
 		char *auditlog_filename;
 		int auditlog_filesize;
 		// SSL related, proxy to server
@@ -456,6 +465,7 @@ class MySQL_Threads_Handler
 		bool stats_time_query_processor;
 		bool query_cache_stores_empty_result;
 		bool kill_backend_connection_when_disconnect;
+		bool client_session_track_gtid;
 	} variables;
 	struct {
 		unsigned int mirror_sessions_current;
@@ -527,10 +537,15 @@ class MySQL_Threads_Handler
 	unsigned long long get_max_connect_timeout();
 	unsigned long long get_unexpected_com_quit();
 	unsigned long long get_unexpected_packet();
+	unsigned long long get_hostgroup_locked();
+	unsigned long long get_hostgroup_locked_set_cmds();
+	unsigned long long get_hostgroup_locked_queries();
 	unsigned long long get_aws_aurora_replicas_skipped_during_query();
 	unsigned long long get_backend_lagging_during_query();
 	unsigned long long get_backend_offline_during_query();
 	unsigned long long get_queries_with_max_lag_ms();
+	unsigned long long get_queries_with_max_lag_ms__delayed();
+	unsigned long long get_queries_with_max_lag_ms__total_wait_time_us();
 	unsigned long long get_killed_connections();
 	unsigned long long get_killed_queries();
 	iface_info *MLM_find_iface_from_fd(int fd) {
