@@ -50,6 +50,7 @@ class MySQL_Connection {
 		unsigned int compression_min_length;
 		char *init_connect;
 		bool init_connect_sent;
+		bool sql_mode_sent;
 		char *ldap_user_variable;
 		char *ldap_user_variable_value;
 		bool ldap_user_variable_sent;
@@ -200,5 +201,7 @@ class MySQL_Connection {
 
 	bool get_gtid(char *buff, uint64_t *trx_id);
 	void reduce_auto_increment_delay_token() { if (auto_increment_delay_token) auto_increment_delay_token--; };
+
+	bool match_tracked_options(MySQL_Connection *c);
 };
 #endif /* __CLASS_MYSQL_CONNECTION_H */
