@@ -11,7 +11,7 @@
 #define PROXYSQL_ENUMS
 
 enum log_event_type {
-	PROXYSQL_QUERY,
+	PROXYSQL_COM_QUERY,
 	PROXYSQL_MYSQL_AUTH_OK,
 	PROXYSQL_MYSQL_AUTH_ERR,
 	PROXYSQL_MYSQL_AUTH_CLOSE,
@@ -26,7 +26,9 @@ enum log_event_type {
 	PROXYSQL_SQLITE_AUTH_OK,
 	PROXYSQL_SQLITE_AUTH_ERR,
 	PROXYSQL_SQLITE_AUTH_CLOSE,
-	PROXYSQL_SQLITE_AUTH_QUIT
+	PROXYSQL_SQLITE_AUTH_QUIT,
+	PROXYSQL_COM_STMT_EXECUTE,
+	PROXYSQL_COM_STMT_PREPARE
 };
 
 enum cred_username_type { USERNAME_BACKEND, USERNAME_FRONTEND };
@@ -642,6 +644,7 @@ __thread int mysql_thread___connect_timeout_server;
 __thread int mysql_thread___connect_timeout_server_max;
 __thread int mysql_thread___query_processor_iterations;
 __thread int mysql_thread___query_processor_regex;
+__thread int mysql_thread___set_query_lock_on_hostgroup;
 __thread int mysql_thread___reset_connection_algorithm;
 __thread uint32_t mysql_thread___server_capabilities;
 __thread int mysql_thread___auto_increment_delay_multiplex;
@@ -661,17 +664,19 @@ __thread bool mysql_thread___servers_stats;
 __thread bool mysql_thread___commands_stats;
 __thread bool mysql_thread___query_digests;
 __thread bool mysql_thread___query_digests_lowercase;
+__thread bool mysql_thread___query_digests_replace_null;
 __thread bool mysql_thread___query_digests_normalize_digest_text;
 __thread bool mysql_thread___query_digests_track_hostname;
 __thread int mysql_thread___query_digests_max_digest_length;
 __thread int mysql_thread___query_digests_max_query_length;
 __thread int mysql_thread___show_processlist_extended;
+__thread int mysql_thread___session_idle_ms;
+__thread int mysql_thread___hostgroup_manager_verbose;
 __thread bool mysql_thread___default_reconnect;
 __thread bool mysql_thread___session_idle_show_processlist;
 __thread bool mysql_thread___sessions_sort;
-__thread bool mysql_thread___session_idle_ms;
-__thread int mysql_thread___hostgroup_manager_verbose;
 __thread bool mysql_thread___kill_backend_connection_when_disconnect;
+__thread bool mysql_thread___client_session_track_gtid;
 
 /* variables used for Query Cache */
 __thread int mysql_thread___query_cache_size_MB;
@@ -685,6 +690,8 @@ __thread char * mysql_thread___ssl_p2s_cipher;
 /* variables used by events log */
 __thread char * mysql_thread___eventslog_filename;
 __thread int mysql_thread___eventslog_filesize;
+__thread int mysql_thread___eventslog_default_log;
+__thread int mysql_thread___eventslog_format;
 
 /* variables used by audit log */
 __thread char * mysql_thread___auditlog_filename;
@@ -771,6 +778,7 @@ extern __thread int mysql_thread___connect_timeout_server;
 extern __thread int mysql_thread___connect_timeout_server_max;
 extern __thread int mysql_thread___query_processor_iterations;
 extern __thread int mysql_thread___query_processor_regex;
+extern __thread int mysql_thread___set_query_lock_on_hostgroup;
 extern __thread int mysql_thread___reset_connection_algorithm;
 extern __thread uint32_t mysql_thread___server_capabilities;
 extern __thread int mysql_thread___auto_increment_delay_multiplex;
@@ -790,17 +798,19 @@ extern __thread bool mysql_thread___servers_stats;
 extern __thread bool mysql_thread___commands_stats;
 extern __thread bool mysql_thread___query_digests;
 extern __thread bool mysql_thread___query_digests_lowercase;
+extern __thread bool mysql_thread___query_digests_replace_null;
 extern __thread bool mysql_thread___query_digests_normalize_digest_text;
 extern __thread bool mysql_thread___query_digests_track_hostname;
 extern __thread int mysql_thread___query_digests_max_digest_length;
 extern __thread int mysql_thread___query_digests_max_query_length;
 extern __thread int mysql_thread___show_processlist_extended;
+extern __thread int mysql_thread___session_idle_ms;
+extern __thread int mysql_thread___hostgroup_manager_verbose;
 extern __thread bool mysql_thread___default_reconnect;
 extern __thread bool mysql_thread___session_idle_show_processlist;
 extern __thread bool mysql_thread___sessions_sort;
-extern __thread bool mysql_thread___session_idle_ms;
-extern __thread int mysql_thread___hostgroup_manager_verbose;
 extern __thread bool mysql_thread___kill_backend_connection_when_disconnect;
+extern __thread bool mysql_thread___client_session_track_gtid;
 
 /* variables used for Query Cache */
 extern __thread int mysql_thread___query_cache_size_MB;
@@ -814,6 +824,8 @@ extern __thread char * mysql_thread___ssl_p2s_cipher;
 /* variables used by events log */
 extern __thread char * mysql_thread___eventslog_filename;
 extern __thread int mysql_thread___eventslog_filesize;
+extern __thread int mysql_thread___eventslog_default_log;
+extern __thread int mysql_thread___eventslog_format;
 
 /* variables used by audit log */
 extern __thread char * mysql_thread___auditlog_filename;
