@@ -25,6 +25,8 @@ typedef uint8_t uchar;
 #define CLIENT_PLUGIN_AUTH_LENENC_CLIENT_DATA 0x00200000
 #endif
 
+extern const MARIADB_CHARSET_INFO * proxysql_find_charset_nr(unsigned int nr);
+
 #ifdef DEBUG
 static void __dump_pkt(const char *func, unsigned char *_ptr, unsigned int len) {
 
@@ -503,7 +505,7 @@ bool MySQL_Protocol::generate_pkt_EOF(bool send, void **ptr, unsigned int *len,
 			case PROXYSQL_SESSION_SQLITE:
 			case PROXYSQL_SESSION_ADMIN:
 			case PROXYSQL_SESSION_STATS:
-				internal_status += SERVER_STATUS_NO_BACKSLASH_ESCAPES;
+				internal_status |= SERVER_STATUS_NO_BACKSLASH_ESCAPES;
 				break;
 			default:
 				break;
@@ -511,7 +513,7 @@ bool MySQL_Protocol::generate_pkt_EOF(bool send, void **ptr, unsigned int *len,
 	}
 	if (*myds && (*myds)->myconn) {
 		if ((*myds)->myconn->options.no_backslash_escapes) {
-			internal_status += SERVER_STATUS_NO_BACKSLASH_ESCAPES;
+			internal_status |= SERVER_STATUS_NO_BACKSLASH_ESCAPES;
 		}
 		(*myds)->pkt_sid=sequence_id;
 	}
@@ -607,14 +609,40 @@ bool MySQL_Protocol::generate_pkt_OK(bool send, void **ptr, unsigned int *len, u
 	char msg_prefix;
 	uint8_t msg_len_len=mysql_encode_length(msg_len, &msg_prefix);
 
+	bool client_session_track=false;
+	//char gtid_buf[128];
+	char gtid_prefix;
+	uint8_t gtid_len=0;
+	uint8_t gtid_len_len=0;
+
 	mysql_hdr myhdr;
 	myhdr.pkt_id=sequence_id;
 	myhdr.pkt_length=1+affected_rows_len+last_insert_id_len+sizeof(uint16_t)+sizeof(uint16_t)+msg_len;
 	if (msg_len) myhdr.pkt_length+=msg_len_len;
-  unsigned int size=myhdr.pkt_length+sizeof(mysql_hdr);
-  unsigned char *_ptr=(unsigned char *)l_alloc(size);
-  memcpy(_ptr, &myhdr, sizeof(mysql_hdr));
-  int l=sizeof(mysql_hdr);
+
+	if (*myds && (*myds)->myconn) {
+		if ((*myds)->myconn->options.client_flag & CLIENT_SESSION_TRACKING) {
+			if (mysql_thread___client_session_track_gtid) {
+				if (sess) {
+					if (sess->gtid_hid >= 0) {
+						myhdr.pkt_length++;
+						client_session_track=true;
+						gtid_len = strlen(sess->gtid_buf);
+						gtid_len_len = mysql_encode_length(gtid_len, &gtid_prefix);
+						myhdr.pkt_length += gtid_len_len;
+						myhdr.pkt_length += gtid_len;
+						myhdr.pkt_length += 4; // headers related to GTID
+					}
+				}
+			}
+		}
+	}
+
+
+	unsigned int size=myhdr.pkt_length+sizeof(mysql_hdr);
+	unsigned char *_ptr=(unsigned char *)l_alloc(size);
+	memcpy(_ptr, &myhdr, sizeof(mysql_hdr));
+	int l=sizeof(mysql_hdr);
 	_ptr[l]=0x00; l++;
 	l+=write_encoded_length(_ptr+l, affected_rows, affected_rows_len, affected_rows_prefix);
 	l+=write_encoded_length(_ptr+l, last_insert_id, last_insert_id_len, last_insert_id_prefix);
@@ -624,15 +652,19 @@ bool MySQL_Protocol::generate_pkt_OK(bool send, void **ptr, unsigned int *len, u
 			case PROXYSQL_SESSION_SQLITE:
 			case PROXYSQL_SESSION_ADMIN:
 			case PROXYSQL_SESSION_STATS:
-				internal_status += SERVER_STATUS_NO_BACKSLASH_ESCAPES;
+				internal_status |= SERVER_STATUS_NO_BACKSLASH_ESCAPES;
 				break;
 			default:
 				break;
 		}
+		if (sess->session_type == PROXYSQL_SESSION_MYSQL) {
+			sess->CurrentQuery.have_affected_rows = true;
+			sess->CurrentQuery.affected_rows = affected_rows;
+		}
 	}
 	if (*myds && (*myds)->myconn) {
 		if ((*myds)->myconn->options.no_backslash_escapes) {
-			internal_status += SERVER_STATUS_NO_BACKSLASH_ESCAPES;
+			internal_status |= SERVER_STATUS_NO_BACKSLASH_ESCAPES;
 		}
 	}
 	memcpy(_ptr+l, &internal_status, sizeof(uint16_t)); l+=sizeof(uint16_t);
@@ -641,6 +673,21 @@ bool MySQL_Protocol::generate_pkt_OK(bool send, void **ptr, unsigned int *len, u
 		l+=write_encoded_length(_ptr+l, msg_len, msg_len_len, msg_prefix);
 		memcpy(_ptr+l, msg, msg_len);
 	}
+	l+=msg_len;
+	if (client_session_track == true) {
+		_ptr[l]=0x00; l++;
+		if (gtid_len) {
+			unsigned char gtid_prefix_h1 = gtid_len+2;
+			unsigned char state_change_prefix = gtid_prefix_h1+2;
+			_ptr[l] = state_change_prefix; l++;
+			_ptr[l]=0x03; l++; // SESSION_TRACK_GTIDS
+			_ptr[l] = gtid_prefix_h1; l++;
+			_ptr[l]=0x00; l++;
+			// l+=write_encoded_length(_ptr+l, gtid_len, gtid_len_len, gtid_prefix); // overcomplicated
+			_ptr[l] = gtid_len; l++;
+			memcpy(_ptr+l, sess->gtid_buf, gtid_len);
+		}
+	}
 	if (send==true) {
 		(*myds)->PSarrayOUT->add((void *)_ptr,size);
 		switch ((*myds)->DSS) {
@@ -1157,7 +1204,7 @@ bool MySQL_Protocol::generate_pkt_initial_handshake(bool send, void **ptr, unsig
   memcpy(_ptr+l,&mysql_thread___server_capabilities, sizeof(mysql_thread___server_capabilities)/2); l+=sizeof(mysql_thread___server_capabilities)/2;
   memcpy(_ptr+l,&mysql_thread___default_charset, sizeof(mysql_thread___default_charset)); l+=sizeof(mysql_thread___default_charset);
   memcpy(_ptr+l,&server_status, sizeof(server_status)); l+=sizeof(server_status);
-  memcpy(_ptr+l,"\x0f\x80\x15",3); l+=3;
+  memcpy(_ptr+l,"\x8f\x80\x15",3); l+=3;
   for (i=0;i<10; i++) { _ptr[l]=0x00; l++; } //filler
   //create_random_string(mypkt->data+l,12,(struct my_rnd_struct *)&rand_st); l+=12;
 //#ifdef MARIADB_BASE_VERSION
@@ -1629,6 +1676,15 @@ bool MySQL_Protocol::process_pkt_handshake_response(unsigned char *pkt, unsigned
 
 __do_auth:
 
+	{
+		// reject connections from unknown charsets
+		const MARIADB_CHARSET_INFO * c = proxysql_find_charset_nr(charset);
+		if (!c) {
+			proxy_error("Client %s:%d is trying to use unknown charset %d. Disconnecting\n", (*myds)->addr.addr, (*myds)->addr.port, charset);
+			ret = false;
+			goto __exit_do_auth;
+		}
+	}
 	if (session_type == PROXYSQL_SESSION_CLICKHOUSE) {
 #ifdef PROXYSQLCLICKHOUSE
 		password=GloClickHouseAuth->lookup((char *)user, USERNAME_FRONTEND, &_ret_use_ssl, &default_hostgroup, &default_schema, &schema_locked, &transaction_persistent, &fast_forward, &max_connections, &sha1_pass);
@@ -1787,6 +1843,8 @@ __do_auth:
 		}
 	}
 
+__exit_do_auth:
+
 	if (_ret_use_ssl==true) {
 		(*myds)->sess->use_ssl = true;
 	}
@@ -2106,6 +2164,52 @@ stmt_execute_metadata_t * MySQL_Protocol::get_binds_from_pkt(void *ptr, unsigned
 	return ret;
 }
 
+bool MySQL_Protocol::generate_COM_QUERY_from_COM_FIELD_LIST(PtrSize_t *pkt) {
+	PtrSize_t n_pkt;
+	unsigned int o_pkt_size = pkt->size;
+	char *pkt_ptr = (char *)pkt->ptr;
+
+	pkt_ptr+=5;
+	// some sanity check
+	void *a = NULL;
+	a = memchr((void *)pkt_ptr, 0, o_pkt_size-5);
+	if (a==NULL) return false; // we failed to parse
+	char *tablename = strdup(pkt_ptr);
+	unsigned int wild_len = o_pkt_size - 5 - strlen(tablename) - 1;
+	char *wild = NULL;
+	if (wild_len > 0) {
+		pkt_ptr+=strlen(tablename);
+		pkt_ptr++;
+		wild=strndup(pkt_ptr,wild_len);
+	}
+	char *q = NULL;
+	if ((*myds)->com_field_wild) {
+		free((*myds)->com_field_wild);
+		(*myds)->com_field_wild=NULL;
+	}
+	if (wild) {
+		(*myds)->com_field_wild=strdup(wild);
+	}
+
+	char *qt = (char *)"SELECT * FROM %s WHERE 1=0";
+	q = (char *)malloc(strlen(qt)+strlen(tablename));
+	sprintf(q,qt,tablename);
+	l_free(pkt->size, pkt->ptr);
+	pkt->size = strlen(q)+5;
+	mysql_hdr Hdr;
+	Hdr.pkt_id=1;
+	Hdr.pkt_length = pkt->size - 4;
+	pkt->ptr=malloc(pkt->size);
+	memcpy(pkt->ptr,&Hdr,sizeof(mysql_hdr));
+    memset((char *)pkt->ptr+4,3,1); // COM_QUERY
+    memcpy((char *)pkt->ptr+5,q,pkt->size-5);
+
+	if (wild) free(wild);
+	free(tablename);
+	free(q);
+	return true;
+}
+
 MySQL_ResultSet::MySQL_ResultSet() {
 	buffer = NULL;
 	//reset_pid = true;
@@ -2115,6 +2219,7 @@ void MySQL_ResultSet::init(MySQL_Protocol *_myprot, MYSQL_RES *_res, MYSQL *_my,
 	resultset_completed=false;
 	myprot=_myprot;
 	mysql=_my;
+	MySQL_Data_Stream * c_myds = *(myprot->myds);
 	if (buffer==NULL) {
 	//if (_stmt==NULL) { // we allocate this buffer only for not prepared statements
 	// removing the previous assumption. We allocate this buffer also for prepared statements
@@ -2145,17 +2250,25 @@ void MySQL_ResultSet::init(MySQL_Protocol *_myprot, MYSQL_RES *_res, MYSQL *_my,
 	if (myprot==NULL) {
 		return; // this is a mirror
 	}
-	myprot->generate_pkt_column_count(false,&pkt.ptr,&pkt.size,sid,num_fields,this);
-	sid++;
-	//PSarrayOUT->add(pkt.ptr,pkt.size);
-	resultset_size+=pkt.size;
+	if (c_myds->com_field_list==false) {
+		myprot->generate_pkt_column_count(false,&pkt.ptr,&pkt.size,sid,num_fields,this);
+		sid++;
+		resultset_size+=pkt.size;
+	}
 	// columns description
 	for (unsigned int i=0; i<num_fields; i++) {
 		MYSQL_FIELD *field=mysql_fetch_field(result);
-		myprot->generate_pkt_field(false,&pkt.ptr,&pkt.size,sid,field->db,field->table,field->org_table,field->name,field->org_name,field->charsetnr,field->length,field->type,field->flags,field->decimals,false,0,NULL,this);
-		//PSarrayOUT->add(pkt.ptr,pkt.size);
-		resultset_size+=pkt.size;
-		sid++;
+		if (c_myds->com_field_list==false) {
+			myprot->generate_pkt_field(false,&pkt.ptr,&pkt.size,sid,field->db,field->table,field->org_table,field->name,field->org_name,field->charsetnr,field->length,field->type,field->flags,field->decimals,false,0,NULL,this);
+			resultset_size+=pkt.size;
+			sid++;
+		} else {
+			if (c_myds->com_field_wild==NULL || mywildcmp(c_myds->com_field_wild,field->name)) {
+				myprot->generate_pkt_field(false,&pkt.ptr,&pkt.size,sid,field->db,field->table,field->org_table,field->name,field->org_name,field->charsetnr,field->length,field->type,field->flags,field->decimals,true,4,(char *)"null",this);
+				resultset_size+=pkt.size;
+				sid++;
+			}
+		}
 	}
 	// first EOF
 	unsigned int nTrx=myds->sess->NumActiveTransactions();
@@ -2172,9 +2285,11 @@ void MySQL_ResultSet::init(MySQL_Protocol *_myprot, MYSQL_RES *_res, MYSQL *_my,
 		if (RESULTSET_BUFLEN <= (buffer_used + 9)) {
 			buffer_to_PSarrayOut();
 		}
+	if (myds->com_field_list==false) {
 		myprot->generate_pkt_EOF(false, NULL, NULL, sid, 0, setStatus, this);
 		sid++;
 		resultset_size += 9;
+	}
 	//}
 	if (_stmt) { // binary protocol , we also assume we have ALL the resultset
 		buffer_to_PSarrayOut();
@@ -2260,7 +2375,7 @@ unsigned int MySQL_ResultSet::add_row(MYSQL_ROW row) {
 // it assumes that the MYSQL_ROW is an format ready to be sent to the client
 unsigned int MySQL_ResultSet::add_row2(MYSQL_ROWS *row, unsigned char *offset) {
 	unsigned long length=row->length;
-
+	num_rows++;
 	uint8_t pkt_sid=sid;
 	if (length < (0xFFFFFF+sizeof(mysql_hdr))) {
 		mysql_hdr myhdr;
@@ -2344,6 +2459,15 @@ void MySQL_ResultSet::add_err(MySQL_Data_Stream *_myds) {
 	resultset_completed=true;
 }
 
+/*
+bool MySQL_ResultSet::get_COM_FIELD_LIST_response(PtrSizeArray *PSarrayFinal) {
+	transfer_started=true;
+	if (myprot) {
+	}
+	return resultset_completed;
+}
+*/
+
 bool MySQL_ResultSet::get_resultset(PtrSizeArray *PSarrayFinal) {
 	transfer_started=true;
 	if (myprot) {
