@@ -11,6 +11,7 @@ extern __thread int mysql_thread___query_digests_max_query_length;
 #include <ctype.h>
 #define bool char
 extern __thread bool mysql_thread___query_digests_lowercase;
+extern __thread bool mysql_thread___query_digests_replace_null;
 
 void tokenizer(tokenizer_t *result, const char* s, const char* delimiters, int empties )
 {
@@ -207,7 +208,9 @@ char *mysql_query_digest_and_first_comment(char *s, int _len, char **first_comme
 	char fns=0;
 
 	bool lowercase=0;
+	bool replace_null=0;
 	lowercase=mysql_thread___query_digests_lowercase;
+	replace_null = mysql_thread___query_digests_replace_null;
 
 	while(i < len)
 	{
@@ -272,6 +275,35 @@ char *mysql_query_digest_and_first_comment(char *s, int _len, char **first_comme
 					i++;
 					continue;
 				}
+				if (replace_null) {
+				if (*s == 'n' || *s == 'N') { // we search for NULL , #2171
+					if (i && is_token_char(prev_char)) {
+						if (len>=4) {
+							if (i<len-3) {
+								// it is only 4 chars, let's skip strncasecmp
+								//if (strncasecmp(s,"null",4)==0) {
+								if (*(s+1) == 'u' || *(s+1) == 'U') {
+									if (*(s+2) == 'l' || *(s+2) == 'L') {
+										if (*(s+3) == 'l' || *(s+3) == 'L') {
+											if (i==len-3) {
+												*p_r++ = '?';
+												*p_r = 0;
+												return r;
+											} else {
+												if (is_token_char(*(s+4))){
+													*p_r++ = '?';
+													s+=4;
+													i+=4;
+												}
+											}
+										}
+									}
+								}
+							}
+						}
+					}
+				}
+				}
 			}
 		}
 
@@ -509,6 +541,11 @@ char *mysql_query_digest_and_first_comment(char *s, int _len, char **first_comme
 		e--;
 		if (*e==' ') {
 			*e=0;
+			// maybe 2 trailing spaces . It happens with comments
+			e--;
+			if (*e==' ') {
+				*e=0;
+			}
 		}
 	}
 
