@@ -216,10 +216,12 @@ MySQL_Data_Stream::MySQL_Data_Stream() {
 	CompPktOUT.partial=0;
 	multi_pkt.ptr=NULL;
 	multi_pkt.size=0;
-
+	
 	statuses.questions = 0;
 	statuses.myconnpoll_get = 0;
 	statuses.myconnpoll_put = 0;
+
+	com_field_wild=NULL;
 }
 
 // Destructor
@@ -242,6 +244,11 @@ MySQL_Data_Stream::~MySQL_Data_Stream() {
 
 	free_mysql_real_query();
 
+	if (com_field_wild) {
+		free(com_field_wild);
+		com_field_wild=NULL;
+	}
+
 	proxy_debug(PROXY_DEBUG_NET,1, "Shutdown Data Stream. Session=%p, DataStream=%p\n" , sess, this);
 	PtrSize_t pkt;
 	if (PSarrayIN) {
@@ -1382,3 +1389,12 @@ void MySQL_Data_Stream::destroy_queues() {
 	queue_destroy(queueIN);
 	queue_destroy(queueOUT);
 }
+
+void MySQL_Data_Stream::destroy_MySQL_Connection_From_Pool(bool sq) {
+	MySQL_Connection *mc=myconn;
+	mc->last_time_used=sess->thread->curtime;
+	detach_connection();
+	unplug_backend();
+	mc->send_quit=sq;
+	MyHGM->destroy_MyConn_from_pool(mc);
+}
